module code_param
    implicit none
    integer, parameter :: dp = kind(1.0d0)  ! 双精度
    real(dp),parameter::pi = acos(-1.0_dp)
    complex(dp),parameter::im = (0.,1.)                 !   Imagine unit  
    integer Nk,numk_bz
    real(dp),allocatable::BZklist(:,:)
    real(dp) m0,t1,ax,ay,mu
    real(dp) omega
    parameter(m0 = 1.0,t1 = 1.0,ax = 1.0,ay = 1.0,Nk = 1e2, mu = 0.0, omega = 1e-8)
end module code_param
!==============================================================================================================================================================
program chern_insulator_hall_conductance
    use code_param
    implicit none
    integer :: ik0
    real(dp) :: kx, ky, hall_conductance
    real(dp),external :: calculate_berry_curvature
    call squareBZ()  ! 构建布里渊区

    hall_conductance = 0.0

    ! 遍历 k 空间，计算 Berry 曲率并积分
    do ik0 = 1,size(BZklist,2)
        kx = BZklist(1,ik0)
        ky = BZklist(2,ik0)
        hall_conductance = hall_conductance + calculate_berry_curvature(kx,ky,1) * (pi/Nk)**2
    end do

    ! 输出结果
    write(*,"(A50,F15.6)")"Hall Conductance (in units of e^2/h): ", hall_conductance / (2.0 * pi)

end program chern_insulator_hall_conductance
!==============================================================================================================================================================
    function calculate_berry_curvature(kx,ky,ind_band)
    ! 计算某一条能带的 Berry 曲率的子程序
    use code_param
    implicit none
    integer,parameter::hn = 2  ! 哈密顿量维度
    integer ie1,ie2
    real(dp), intent(in) :: kx, ky
    integer, intent(in) :: ind_band
    real(dp) :: eigvals(hn),calculate_berry_curvature
    complex(dp) :: re1(hn), dHdkx(hn,hn), dHdky(hn,hn), Ham(hn,hn), eigvecs(hn,hn)
    complex(dp),external::Ave_Ham

    ! H = (m0 - tx cos(kx) - ty cos(ky))\sigma_z + ax sin(kx) \sigma_x + ay * sin(ky) * sigma_y
    Ham = 0.0
    Ham(1,1) = m0 - t1 * (cos(kx) + cos(ky)) - mu
    Ham(2,2) = -(m0 - t1 * (cos(kx) + cos(ky))) - mu
    Ham(1,2) = ax * sin(kx) - im * ay * sin(ky)
    Ham(2,1) = ax * sin(kx) + im * ay * sin(ky)
    
    ! 计算哈密顿量的特征值和特征向量
    eigvecs = 0.0
    eigvals = 0.0
    call diagonalize_hermitian_matrix(hn, Ham, eigvecs, eigvals)

    ! 构建哈密顿量对 kx 和 ky 的导数矩阵
    dHdkx = 0.0
    dHdky = 0.0

    !  DH_x
    dHdkx(1,1) = t1 * sin(kx)
    dHdkx(2,2) = -t1 * sin(kx)

    dHdkx(1,2) = ax * cos(kx) 
    dHdkx(2,1) = ax * cos(kx)
    
    !  DH_y
    dHdky(1,1) = t1 * sin(ky)
    dHdky(2,2) = -t1 * sin(ky)

    dHdky(1,2) = -im * ay * cos(ky)
    dHdky(2,1) = im * ay * cos(ky)

    ! 计算 Berry 曲率
    re1 = 0.0
    do ie1 = 1, hn   ! 索引能带
    do ie2 = 1, hn
        if (ie2 /= ie1) then
            re1(ie1) = re1(ie1) + (Ave_Ham(hn, eigvecs(:, ie1), dHdkx, eigvecs(:, ie2)) * &
                            Ave_Ham(hn, eigvecs(:, ie2), dHdky, eigvecs(:, ie1))) / &
                            ((eigvals(ie1) - eigvals(ie2))**2 + omega)  ! 添加正则项防止除零
        endif
    end do
    end do
    calculate_berry_curvature = 2 * aimag(re1(ind_band))
end function calculate_berry_curvature
!==============================================================================================================================================================
function Ave_Ham(matdim,psi1,Ham,psi2)
    ! 计算  <psi_1|Ham|psi_2>
    use code_param
    implicit none
    integer i0,j0,matdim
    complex(dp) Ave_Ham,expectation,Ham(matdim,matdim),temp(matdim),psi1(matdim),psi2(matdim)
    expectation = 0.0
    do i0 = 1,matdim
        temp(i0) = 0.0
        do j0 = 1,matdim
            temp(i0) = temp(i0) + Ham(i0, j0) * psi2(j0)
        end do
    end do

    do i0 = 1,matdim
        expectation = expectation + conjg(psi1(i0)) * temp(i0)
    end do
    Ave_Ham = expectation
    return
end function 
!============================================================================================================================
subroutine squareBZ()
    ! 构建四方BZ
    use code_param
    integer ikx,iky,i0
    ! 对于四方点阵,BZ的点数可以直接确定
    numk_bz = (2 * Nk)**2   
    allocate(BZklist(2,numk_bz))
    i0 = 0
    open(30,file = "BZ.dat")
    do ikx = -Nk,Nk - 1
        do iky = -Nk,Nk - 1
            i0 = i0 + 1
            BZklist(1,i0) = pi * ikx/(1.0 * Nk)
            BZklist(2,i0) = pi * iky/(1.0 * Nk) 
            write(30,"(4F15.8)")BZklist(1,i0),BZklist(2,i0)
        end do
    end do
    close(30)
    return  
end subroutine
!==============================================================================================================================================================
subroutine diagonalize_Hermitian_matrix(matdim, matin, matout, mateigval)
    ! 厄米矩阵对角化
    ! matin 输入矩阵, matout 本征矢量, mateigval 本征值
    implicit none
    integer, parameter :: dp = kind(1.0d0)  ! 双精度
    integer, intent(in) :: matdim
    integer :: lda0, lwmax0, lwork, lrwork, liwork, info
    complex(dp), intent(in) :: matin(matdim, matdim)
    complex(dp), intent(out) :: matout(matdim, matdim)
    real(dp), intent(out) :: mateigval(matdim)
    complex(dp), allocatable :: work(:)
    real(dp), allocatable :: rwork(:)
    integer, allocatable :: iwork(:)
    !-----------------
    lda0 = matdim
    lwmax0 = 2 * matdim + matdim**2
    allocate(work(lwmax0))
    allocate(rwork(1 + 5 * matdim + 2 * matdim**2))
    allocate(iwork(3 + 5 * matdim))

    matout = matin
    lwork = -1
    liwork = -1
    lrwork = -1

    ! 初次调用 zheevd 获取最佳工作空间大小
    call zheevd('V', 'U', matdim, matout, lda0, mateigval, work, lwork, rwork, lrwork, iwork, liwork, info)

    ! 根据第一次调用的返回值重新调整工作空间大小
    lwork = min(2 * matdim + matdim**2, int(work(1)))
    lrwork = min(1 + 5 * matdim + 2 * matdim**2, int(rwork(1)))
    liwork = min(3 + 5 * matdim, iwork(1))

    ! 重新分配工作空间
    deallocate(work)
    allocate(work(lwork))
    deallocate(rwork)
    allocate(rwork(lrwork))
    deallocate(iwork)
    allocate(iwork(liwork))

    ! 第二次调用 zheevd 计算本征值和本征矢量
    call zheevd('V', 'U', matdim, matout, lda0, mateigval, work, lwork, rwork, lrwork, iwork, liwork, info)

    ! 错误处理
    if (info > 0) then
        open(11, file = "mes.dat", status = "unknown")
        write(11, *) 'The algorithm failed to compute eigenvalues.'
        close(11)
    end if

    return
end subroutine diagonalize_Hermitian_matrix